'use strict';

//###################################################
//                  Async calls
//###################################################

var currentRequests = {};
function setRequestProcessing(uuid, requestObj) {
    if (currentRequests[uuid] == null || currentRequests[uuid].state == null) {
        currentRequests[uuid] = { ts: new Date().getTime(), state: 'processing', req: requestObj };
        return;
    }
    if (currentRequests[uuid].state != 'success' && currentRequests[uuid].state != 'error' && currentRequests[uuid].state != 'cancelled')
        currentRequests[uuid].state = 'processing';
}
function updateCurrentResponseObj(uuid, responseObj) {
    if (currentRequests[uuid] == null)
        return;
    currentRequests[uuid].result = responseObj;
}
function setRequestSuccess(uuid, responseObj) {
    if (currentRequests[uuid] == null) {
        currentRequests[uuid] = { ts: new Date().getTime(), state: '' };
    }
    currentRequests[uuid].state = 'success';
    currentRequests[uuid].result = responseObj;
}
function setRequestError(uuid, message, responseObj) {
    if (currentRequests[uuid] == null) {
        currentRequests[uuid] = { ts: new Date().getTime(), state: '', message: '' };
    }
    currentRequests[uuid].state = 'error';
    currentRequests[uuid].message = message;
    currentRequests[uuid].result = responseObj;
}
function setRequestCancelled(uuid, responseObj) {
    if (currentRequests[uuid] == null || currentRequests[uuid].state == null) {
        currentRequests[uuid] = { ts: new Date().getTime(), state: 'cancelled' };
        return;
    }

    if (currentRequests[uuid].state != 'error' && currentRequests[uuid].state != 'success') {
        currentRequests[uuid].state = 'cancelled';
        currentRequests[uuid].result = responseObj;
    }
}

function cleanOlderThan(retention) {
    var newcurrentRequests = {};
    const currentts = new Date().getTime();
    for (const [key, value] of Object.entries(currentRequests)) {

        var retention_offset = value.ts + retention;
        if (retention_offset < currentts && value.state != 'processing') {    //jelsi minelo 5 minut oraz inprogress jets na false
            var objtext = JSON.stringify(value);
            logger.logger.debug(`Removing expired ${key} from the queue -> ${objtext}`);
        } else {
            newcurrentRequests[key] = value;
        }
    }

    currentRequests = newcurrentRequests;
}

function getCurrentRequests() {
    return currentRequests;
}

module.exports.setRequestProcessing = setRequestProcessing;
module.exports.updateCurrentResponseObj = updateCurrentResponseObj;
module.exports.setRequestSuccess = setRequestSuccess;
module.exports.setRequestError = setRequestError;
module.exports.setRequestCancelled = setRequestCancelled;
module.exports.cleanOlderThan = cleanOlderThan;
module.exports.getCurrentRequests = getCurrentRequests;
