'use strict';

const express = require('express');
const bodyParser = require('body-parser');
const logger = require('./logger');
const Joi = require('joi');

//console.log('aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa');
const apitools = require('./restapi-tools');
//console.log('aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa');
const apiasyncreqs = require('./restapi-asyncreqs');

var nativeTest = null;  // inicjalizowane w metodzie initModule()
var apidef = null;      // inicjalizowane w metodzie initModule()

const yaml_config = require('node-yaml-config');

var readingFromTransactionInProgress = false;
var scheduledCommands_ = [];

//###################################################
//                  Config
//###################################################
logger.logger.info('Loading config, make sure config.yml has valid structure');
//var buildVersion = null;
var config = yaml_config.load(__dirname + '/config.yml');
logger.logger.info('config: DONE');

function ingenicoTransactionSchema() {
    return Joi.object().keys({
        type: Joi.string().required().valid("purchase",
            "purchase_cashback",
            "cash_from_pos",
            "preauthorization",
            "completion",
            "refund",
            "activate_service_mode",
            "deactivate_service_mode",
            "card_verification",
            "preauthorization_increase",
            "preauthorization_reduce",
            "preauthorization_cancel",
            "fleet_card_sale",
            "loyalty_card",
            "card_activation",
            "replenishment",
            "balance_inquiry",
            "purchase_with_registration",
            "card_registration",
            "purchase_adjustment",
            "cash_deposit",
            "no_show",
            "delayed_charges"),
        transactionAmount: Joi.number().required(),
        cashbackAmount: Joi.number().required(),
        tipAmount: Joi.number().required(),
        surchargeFeeAmount: Joi.number().required(),
        foreignGoodsAmount: Joi.number().required(),
        currencyCode: Joi.number().required(),
        reversal: Joi.boolean().required(),
        trackData: Joi.string().required().allow(''),
        transport: ingenicoTransportSchema(),
        callback: Joi.object().keys({
            uri: Joi.string().required(),
            method: Joi.string().required().valid('POST', 'GET', 'PUT')
        }),
        posnetserver: Joi.object(),
        extras: ingenicoExtrasSchema(),
        extend: ingenicoTransactionExtendedSchema()
    });
}

function ingenicoTransportSchema() {
    return Joi.object().keys({
        encryptSession: Joi.boolean().required(),
        encryptAlgorithm: Joi.string(),
        sessionKeyLen: Joi.number(),
        publicKey: Joi.string().allow('')
    });
}

function ingenicoExtrasSchema() {
    return Joi.object().keys({
        responseType: Joi.string().required().valid('simple', 'extended_full', 'extended_short'),
        date: Joi.string().required(),
        version: Joi.string().required(),
        manufacturerName: Joi.string().required(),
        protocolVersion: Joi.string().required()
    });
}

function ingenicoTransactionExtendedSchema() {
    return Joi.object().keys({
        cardData: Joi.string().required().allow(''),
        cardType: Joi.string().required().valid('','fleet_card', 'loyalty_card'),
        cardOrderType: Joi.string().max(3).required().allow(''),
        //trackData: Joi.string().required().allow(''),
        //reversal: Joi.boolean().required(),
        commercialCode: Joi.string().max(3).required().allow(''),
        caldholderId: Joi.string().min(6).max(40).required().allow(''),
        clerkId: Joi.string().min(0).max(25).required().allow(''),
        profileId: Joi.number().required(),
        maxCashbackAmount: Joi.number().required(),
        authorizationCode: Joi.string().max(6).required().allow(''),
        ecrTransactionId: Joi.string().min(2).max(6).required().allow(''),
        privateCardName: Joi.string().min(2).max(12).required().allow(''),
        externalTransactionId: Joi.string().max(36).required().allow(''),
        caldholderPhone: Joi.string().max(15).required().allow(''),
        caldholderEmail: Joi.string().max(50).required().allow(''),
        partialApprovalFlag: Joi.boolean().required(),
        voidTransactionId: Joi.number().required(),
        saleItemList: Joi.array().items(Joi.object().keys({
            itemId: Joi.string().max(16).required(),
            quantity: Joi.number().required(),
            amount: Joi.number().required(),
            settlementIdentifier: Joi.number().required()
        }))
    });
}

function verifyTermResponse(req, res, result, uuid /*optional*/) {
    if (result.terminal.returnCode != 0) {
        if (res != null)
            apitools.sendErrorResponse(req, res, result);

        var closeresult = apitools.ConnectionEnd();
        /*if (!result.ok)
            return apitools.sendErrorResponse(req, res, result);
        else
            return apitools.sendResponse(req, res, msg0);*/
        if (uuid != null)
            apiasyncreqs.setRequestError(uuid, result.error, result);

        return false;
    } else {
        return true;
    }
}

function processPolcardTransactionStatus(req, res) {
    var fulldebug = apitools.isFullDebugMode(req);
    const msg0 = nativeTest.PolcardTransactionStatus(fulldebug ? "1" : "0");
    if (!verifyTermResponse(req, res, msg0)) return;

    return apitools.sendResponse(req, res, msg0);
}

function processPolcardTransactionStatuses(req, res) {
    var fulldebug = apitools.isFullDebugMode(req);
    const msg0 = nativeTest.PolcardTransactionStatuses(fulldebug ? "1" : "0");
    if (!verifyTermResponse(req, res, msg0)) return;

    return apitools.sendResponse(req, res, msg0);
}

function processPolcardIngenicoTransactionEnd(req, res) {
    var fulldebug = apitools.isFullDebugMode(req);
    var resetstate = apitools.isResetStateSet(req);

    var trEnd = nativeTest.PolcardTransactionEnd(fulldebug ? "1" : "0", resetstate?"1":"0");
    //if (!verifyTermResponse(req, res, trEnd)) return;
    logger.logger.info('trEnd done: ' + JSON.stringify(trEnd));

    return apitools.sendResponse(req, res, trEnd);
}

function processPolcardIngenicoTransactionLast(req, res) {
    var fulldebug = apitools.isFullDebugMode(req);
    //var resetstate = apitools.isResetStateSet(req);

    apitools.ConnectionBegin(req, async (err) => {
        if (!err.ok) {
            apitools.sendErrorResponse(req, res, err);
            return;
        }

        var trLast = nativeTest.PolcardTransactionLast(fulldebug ? "1" : "0");
        //if (!verifyTermResponse(req, res, trEnd)) return;
        logger.logger.info('trLast done: ' + JSON.stringify(trLast));

        if (!verifyTermResponse(req, res, trLast)) return;

        //zwracam odpowiedz z API, ale process dalej nasluchuje na dane przychdozace z terminala
        if (!trLast.ok)
            apitools.sendErrorResponse(req, res, trLast);
        else
            apitools.sendResponse(req, res, trLast);

        apitools.ConnectionEnd();
    });

}

function executePolcardIngenicoTransactionAbort(fulldebug) {
    //------------------------
    var keycode = '0';
    var replyType = 99;
    const trReconInfo = nativeTest.PolcardTransactionDataInput(fulldebug ? "1" : "0", { text: keycode, type: replyType, waitForResponse: 1 });
    return trReconInfo;
}
function processPolcardIngenicoTransactionAbort(req, res) {
    var fulldebug = apitools.isFullDebugMode(req);

    if (!apitools.validateSchema(req, res, apidef))
        return;

    const isconnected = apitools.IsConnected();
    if (isconnected)
    {
        const trReconInfo = executePolcardIngenicoTransactionAbort(fulldebug);
        if (!verifyTermResponse(req, res, trReconInfo)) return;

        //------------------------
        var result = apitools.ConnectionEnd();
        if (!result.ok)
            return apitools.sendErrorResponse(req, res, result);
        else
            return apitools.sendResponse(req, res, trReconInfo);

    } else {
        /*apitools.ConnectionBegin(req, (err) => {
            if (!err.ok) {
                apitools.sendErrorResponse(req, res, err);
                return;
            }

            const trReconInfo = executePolcardIngenicoTransactionAbort(fulldebug);
            if (!verifyTermResponse(req, res, trReconInfo)) return;

            //------------------------
            var result = apitools.ConnectionEnd();
            if (!result.ok)
                return apitools.sendErrorResponse(req, res, result);
            else
                return apitools.sendResponse(req, res, trReconInfo);
        });*/
        return apitools.sendErrorResponse(req, res, { ok: false, error: 'Command can be send only when transaction is in progress' });
    }
}

function processPolcardDisplayInfo(req, res) {
    var fulldebug = apitools.isFullDebugMode(req);
    const msg0 = nativeTest.PolcardDisplayInfo(fulldebug ? "1" : "0");
    return apitools.sendResponse(req, res, msg0);
}

function processIngenicoTransactionDataInput(req, res) {
    var fulldebug = apitools.isFullDebugMode(req);

    if (!apitools.validateSchema(req, res, apidef))
        return;

    const isconnected = apitools.IsConnected();

    if (isconnected && readingFromTransactionInProgress)
    {
        scheduleCommand({
            type: 'PolcardTransactionDataInput',
            attributes: [fulldebug ? "1" : "0", { text: req.body.text, type: req.body.type, waitForResponse: 0 }]
        });
        return apitools.sendResponse(req, res, { ok: true });

    } else {
        apitools.ConnectionBegin(req, (err) => {
            if (!err.ok) {
                apitools.sendErrorResponse(req, res, err);
                return;
            }

            //------------------------
            const trReconInfo = nativeTest.PolcardTransactionDataInput(fulldebug ? "1" : "0", { text: req.body.text, type: req.body.type, waitForResponse: 1 });
            if (!verifyTermResponse(req, res, trReconInfo)) return;

            //------------------------
            var result = apitools.ConnectionEnd();
            if (!result.ok)
                return apitools.sendErrorResponse(req, res, result);
            else
                return apitools.sendResponse(req, res, trReconInfo);
        });
    }
}

function scheduleCommand(obj) {
    scheduledCommands_.push(obj);
}
function processIngenicoTransactionSendKey(req, res) {
    var fulldebug = apitools.isFullDebugMode(req);

    if (!apitools.validateSchema(req, res, apidef))
        return;

    const isconnected = apitools.IsConnected();

    //------------------------
    var keycode = '';
    var replyType = 0;
    if (req.body.keycode == 'ok' || req.body.keycode == 'yes') {
        replyType = 1;
        keycode = "0"
    }
    else if (req.body.keycode == 'back' || req.body.keycode == 'no')    //no
    {
        replyType = 1;
        keycode = "1"
    }
    else if (req.body.keycode == 'cancel') {
        replyType = 5;
        keycode = "0"
    }
    else if (req.body.keycode == 'anykey') {
        replyType = 7;
        keycode = "0"
    }

    if (isconnected && readingFromTransactionInProgress)
    {
        scheduleCommand({
            type: 'PolcardTransactionDataInput',
            attributes: [fulldebug ? "1" : "0", { text: keycode, type: replyType, waitForResponse: 0 }]
        });
        //const trReconInfo = nativeTest.PolcardTransactionDataInput(fulldebug ? "1" : "0", { text: keycode, type: replyType, waitForResponse: 0 });  //jesli jestem wtrakcie transackji to terminal akceptuje bufor ale i tak nie odpowaida zadnym innym message
        //if (!verifyTermResponse(req, res, trReconInfo)) return;
        //return apitools.sendResponse(req, res, trReconInfo);

        return apitools.sendResponse(req, res, { ok: true });

    } else {
        apitools.ConnectionBegin(req, (err) => {
            if (!err.ok) {
                apitools.sendErrorResponse(req, res, err);
                return;
            }

            const trReconInfo = nativeTest.PolcardTransactionDataInput(fulldebug ? "1" : "0", { text: keycode, type: replyType, waitForResponse: 1 });  //moge spokojnie ustawic waitForResponse=1 bo i tak powinien zostac zwrocony error message 2830010 (POLCARD_POS_ERROR)
            if (!verifyTermResponse(req, res, trReconInfo)) return;

            //------------------------
            var result = apitools.ConnectionEnd();
            if (!result.ok)
                return apitools.sendErrorResponse(req, res, result);
            else
                return apitools.sendResponse(req, res, trReconInfo);
        });
    }
}

function processIngenicoTransactionSendCode(req, res) {
    var fulldebug = apitools.isFullDebugMode(req);

    if (!apitools.validateSchema(req, res, apidef))
        return;

    const isconnected = apitools.IsConnected();

    //------------------------
    var code = '';
    var replyType = 0;
    if (req.body.type == 'commercial') {
        replyType = 3;
        code = req.body.code;
    }
    if (req.body.type == 'authorization') {
        replyType = 4;
        code = req.body.code;
    }


    if (isconnected && readingFromTransactionInProgress) {
        scheduleCommand({
            type: 'PolcardTransactionDataInput',
            attributes: [fulldebug ? "1" : "0", { text: code, type: replyType, waitForResponse: 0 }]
        });
        return apitools.sendResponse(req, res, { ok: true });

    } else {
        apitools.ConnectionBegin(req, (err) => {
            if (!err.ok) {
                apitools.sendErrorResponse(req, res, err);
                return;
            }

            const trReconInfo = nativeTest.PolcardTransactionDataInput(fulldebug ? "1" : "0", { text: code, type: replyType, waitForResponse: 1 });  //moge spokojnie ustawic waitForResponse=1 bo i tak powinien zostac zwrocony error message 2830010 (POLCARD_POS_ERROR)
            if (!verifyTermResponse(req, res, trReconInfo)) return;

            //------------------------
            var result = apitools.ConnectionEnd();
            if (!result.ok)
                return apitools.sendErrorResponse(req, res, result);
            else
                return apitools.sendResponse(req, res, trReconInfo);
        });
    }
}


function processIngenicoTransactionSendAmount(req, res) {
    var fulldebug = apitools.isFullDebugMode(req);

    if (!apitools.validateSchema(req, res, apidef))
        return;

    const isconnected = apitools.IsConnected();

    //------------------------
    var amount = "0";
    var replyType = 0;
    if (req.body.type == 'cashback') {
        replyType = 2;
        amount = req.body.amount.toString();
    }
    if (req.body.type == 'other') {
        replyType = 2;
        amount = req.body.amount.toString();
    }


    if (isconnected && readingFromTransactionInProgress) {
        scheduleCommand({
            type: 'PolcardTransactionDataInput',
            attributes: [fulldebug ? "1" : "0", { text: amount, type: replyType, waitForResponse: 0 }]
        });
        return apitools.sendResponse(req, res, { ok: true });

    } else {
        apitools.ConnectionBegin(req, (err) => {
            if (!err.ok) {
                apitools.sendErrorResponse(req, res, err);
                return;
            }

            const trReconInfo = nativeTest.PolcardTransactionDataInput(fulldebug ? "1" : "0", { text: amount, type: replyType, waitForResponse: 1 });  //moge spokojnie ustawic waitForResponse=1 bo i tak powinien zostac zwrocony error message 2830010 (POLCARD_POS_ERROR)
            if (!verifyTermResponse(req, res, trReconInfo)) return;

            //------------------------
            var result = apitools.ConnectionEnd();
            if (!result.ok)
                return apitools.sendErrorResponse(req, res, result);
            else
                return apitools.sendResponse(req, res, trReconInfo);
        });
    }
}

function processIngenicoTransactionSendOption(req, res) {
    var fulldebug = apitools.isFullDebugMode(req);

    if (!apitools.validateSchema(req, res, apidef))
        return;

    const isconnected = apitools.IsConnected();

    //------------------------
    var amount = req.body.option.toString();

    if (isconnected && readingFromTransactionInProgress) {
        scheduleCommand({
            type: 'PolcardTransactionDataInput',
            attributes: [fulldebug ? "1" : "0", { text: amount, type: 5, waitForResponse: 0 }]
        });
        return apitools.sendResponse(req, res, { ok: true });

    } else {
        apitools.ConnectionBegin(req, (err) => {
            if (!err.ok) {
                apitools.sendErrorResponse(req, res, err);
                return;
            }

            const trReconInfo = nativeTest.PolcardTransactionDataInput(fulldebug ? "1" : "0", { text: amount, type: 5, waitForResponse: 1 });  //moge spokojnie ustawic waitForResponse=1 bo i tak powinien zostac zwrocony error message 2830010 (POLCARD_POS_ERROR)
            if (!verifyTermResponse(req, res, trReconInfo)) return;

            //------------------------
            var result = apitools.ConnectionEnd();
            if (!result.ok)
                return apitools.sendErrorResponse(req, res, result);
            else
                return apitools.sendResponse(req, res, trReconInfo);
        });
    }
}

function processIngenicoTransactionSendProfile(req, res) {
    var fulldebug = apitools.isFullDebugMode(req);

    if (!apitools.validateSchema(req, res, apidef))
        return;

    const isconnected = apitools.IsConnected();

    //------------------------
    var amount = req.body.profile.toString();

    if (isconnected && readingFromTransactionInProgress) {
        scheduleCommand({
            type: 'PolcardTransactionDataInput',
            attributes: [fulldebug ? "1" : "0", { text: amount, type: 6, waitForResponse: 0 }]
        });
        return apitools.sendResponse(req, res, { ok: true });

    } else {
        apitools.ConnectionBegin(req, (err) => {
            if (!err.ok) {
                apitools.sendErrorResponse(req, res, err);
                return;
            }

            const trReconInfo = nativeTest.PolcardTransactionDataInput(fulldebug ? "1" : "0", { text: amount, type: 6, waitForResponse: 1 });  //moge spokojnie ustawic waitForResponse=1 bo i tak powinien zostac zwrocony error message 2830010 (POLCARD_POS_ERROR)
            if (!verifyTermResponse(req, res, trReconInfo)) return;

            //------------------------
            var result = apitools.ConnectionEnd();
            if (!result.ok)
                return apitools.sendErrorResponse(req, res, result);
            else
                return apitools.sendResponse(req, res, trReconInfo);
        });
    }
}

function processIngenicoTransactionStart(req, res) {

    var fulldebug = apitools.isFullDebugMode(req);
    //var cashRegisterNumber = 0;//getCashRegisterNumber(req);

    if (!apitools.validateSchema(req, res, apidef))
        return;

    apitools.ConnectionBegin(req, async (err) => {
        if (!err.ok) {
            apitools.sendErrorResponse(req, res, err);
            return;
        }

        //------------------------
        //var trStatus = nativeTest.GetTermInfo(fulldebug ? "1" : "0", { cashRegisterNumber: cashRegisterNumber });
        //if (!verifyTermResponse(req, res, trStatus)) return;
        //logger.logger.info('trStatus done: ' + JSON.stringify(trStatus));

        //if (trStatus.terminal.transactionStatus == 13 /* 0x0D TRANSACTION_STATUS_BATCH_COMPLETED*/) {
        //    var rc = ClearBatchCompleted(req, res, fulldebug);
        //    if (!rc) {
        //        apitools.ConnectionEnd();
        //        return;
        //    }
        //}


        //if (trStatus.terminal.transactionStatus == 1 /* 0x01 TRANSACTION_STATUS_WAIT_CARD*/) {
        //    apitools.ConnectionEnd();
        //    apitools.sendErrorResponse(req, res, { ok: false, error: trStatus.terminal.transactionStatusText + ' : Przyu kart lub zamknij rozpoczt transakcj' });
        //    return;
        //}
        //if (trStatus.terminal.transactionStatus == 6 /* 0x06 TRANSACTION_STATUS_WAIT_TREND*/) {
        //    apitools.ConnectionEnd();
        //    apitools.sendErrorResponse(req, res, { ok: false, error: trStatus.terminal.transactionStatusText + ' : Zamknij niedokoczon transakcj' });
        //    return;
        //}

        //------------------------
        //var trEnd = nativeTest.TransactionEnd(fulldebug ? "1" : "0", { cashRegisterNumber: cashRegisterNumber });
        //if (!verifyTermResponse(req, res, trEnd)) return;
        //logger.logger.info('trEnd done: ' + JSON.stringify(trEnd));

        //------------------------
        //trStatus = nativeTest.GetTermInfo(fulldebug ? "1" : "0", { cashRegisterNumber: cashRegisterNumber });
        //if (!verifyTermResponse(req, res, trStatus)) return;
        //logger.logger.info('trStatus done: ' + JSON.stringify(trStatus));

        //------------------------
        logger.logger.info('TransactionBegin - preparing');
        //console.log('!!!!!!!!!!!!!!!! req.body.trackData = ' + req.body.trackData);
        var trpayload = {
            type: req.body.type/* 'purchase'*/,
            /* 
        - "purchase", Sale - 00
        - "purchase_cashback" Sale with Cashback - 09
        - "cash_from_pos" - 12
        - "preauthorization" - 1S
        - "completion" - 1T
        - "refund" - 20
        - "activate_service_mode" - 31
        - "deactivate_service_mode" - 32
        - "card_verification" - 06
        - "preauthorization_increase"- 1U
        - "preauthorization_reduce"  - 1V
        - "preauthorization_cancel"- 1W
        - "fleet_card_sale" - 2F
        - "loyalty_card"  - 3F
        - "card_activation"- 15
        - "replenishment"  - 17
        - "balance_inquiry"- 07 
            else if (type == "purchase_with_registration")
                return "0R";
            else if (type == "card_registration")
                return "1R";
            else if (type == "purchase_adjustment")
                return "11";
            else if (type == "cash_deposit")
                return "28";
            else if (type == "no_show")
                return "0X";
            else if (type == "delayed_charges")
                return "0D";
        */

            transactionAmount: req.body.transactionAmount != null ? req.body.transactionAmount : 0 /*1000*/,
            cashbackAmount: req.body.cashbackAmount != null ? req.body.cashbackAmount : 0 /*1000*/,
            tipAmount: req.body.tipAmount != null ? req.body.tipAmount : 0 /*1000*/,
            surchargeFeeAmount: req.body.surchargeFeeAmount != null ? req.body.surchargeFeeAmount : 0 /*1000*/,
            foreignGoodsAmount: req.body.foreignGoodsAmount != null ? req.body.foreignGoodsAmount : 0 /*1000*/,
            currencyCode: req.body.currencyCode,
            reversal: req.body.reversal ? 1 : 0,
            trackData: req.body.trackData,
            transport: {
                encryptSession: apitools.isBooleanTrue(req.body.transport.encryptSession) ? 1 : 0,
                encryptAlgorithm: req.body.transport.encryptAlgorithm ? req.body.transport.encryptAlgorithm : '',
                sessionKeyLen: req.body.transport.sessionKeyLen ? req.body.transport.sessionKeyLen : 0,
                publicKey: req.body.transport.publicKey ? req.body.transport.publicKey : ''
            },
            extras: req.body.extras ? req.body.extras : null,
            //title: req.body.title != null ? req.body.title : '', //'Purchase 1234'
            //sequenceNb: req.body.sequenceNb != null ? req.body.sequenceNb : '',
            //arguments: req.body.arguments != null ? req.body.arguments : {},
            //cashRegisterNumber: cashRegisterNumber,
            file: apitools.GetLicenseFile()
        };

        if (req.body.extend) {
            // console.log('$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$');
            // console.log(req.body.extend);
            // console.log('$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$');

            trpayload['extend'] = {
                cardData: req.body.extend.cardData,
                cardType: req.body.extend.cardType,
                cardOrderType: req.body.extend.cardOrderType,
                //trackData: req.body.extend.trackData,
                //reversal: (req.body.extend.reversal ? 1 : 0),
                commercialCode: req.body.extend.commercialCode,
                caldholderId: req.body.extend.caldholderId,
                clerkId: req.body.extend.clerkId,
                profileId: req.body.extend.profileId,
                maxCashbackAmount: req.body.extend.maxCashbackAmount,
                authorizationCode: req.body.extend.authorizationCode,
                ecrTransactionId: req.body.extend.ecrTransactionId,
                privateCardName: req.body.extend.privateCardName,
                externalTransactionId: req.body.extend.externalTransactionId,
                caldholderPhone: req.body.extend.caldholderPhone,
                caldholderEmail: req.body.extend.caldholderEmail,
                partialApprovalFlag: (req.body.extend.partialApprovalFlag ? 1 : 0),
                voidTransactionId: req.body.extend.voidTransactionId,
                saleItemList: (req.body.extend.saleItemList ? req.body.extend.saleItemList: [])
            };
        }// else {
        //    console.log('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!');
        //}

        //console.log('0000000000000000000000000000aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa');
        const msg0 = nativeTest.PolcardTransactionBegin(fulldebug ? "1" : "0", trpayload);
        logger.logger.info('msg0 done: ' + JSON.stringify(msg0));
        //console.log('aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa');
        if (!verifyTermResponse(req, res, msg0)) return;

        //zwracam odpowiedz z API, ale process dalej nasluchuje na dane przychdozace z terminala
        if (!msg0.ok)
            apitools.sendErrorResponse(req, res, msg0);
        else
            apitools.sendResponse(req, res, msg0);

        //------------------------
        //var result = apitools.ConnectionEnd();
        //if (!result.ok)
        //    return apitools.sendErrorResponse(req, res, result);
        //else
        //    return apitools.sendResponse(req, res, msg0);

        if (msg0.ok) {
            var msg1 = await startListeningForWaitTrEnd(req, res, fulldebug);
            //console.log('bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb');
            logger.logger.info('finished waiting; msg1 done: ' + JSON.stringify(msg1));

            //TODO - ryzyko ze ktos zawola nativeTest.PolcardTransactionEnd (z resetstate=1) zanim pobiore status ponizej (male ryzyko, ale zawsze) - poprawic w prrzyszlosci
            var trEnd = nativeTest.PolcardTransactionEnd(fulldebug ? "1" : "0", "0");
            //if (!verifyTermResponse(req, res, trEnd)) return;
            logger.logger.info('trEnd done: ' + JSON.stringify(trEnd));

            if (req.body.callback) {
                apitools.notifyBackendAboutTransactionResult(trEnd, req.body.callback);
            }

            if (config.posnetserver.active) {
                if (req.body.posnetserver) {
                    if (trEnd.terminal.authorizationResult == 'base_approval') {
                        apitools.sendToPosnetServer(req.body.posnetserver);
                    } else {
                        logger.logger.warn(`Integracja z PosnetServer jest wlaczona, transakcja na terminalu nie zakonczyla sie sukcesem - paragon nie bedzie drukowany`);
                    }
                } else {
                    logger.logger.info(`Integracja z PosnetServer jest wlaczona, ale request nie zawiera struktury 'posnetserver' ktora mona przekazac do serwisu.`);
                }
            }
        }
        apitools.ConnectionEnd();
    });
}


function processIngenicoPrint(req, res) {
    
    var fulldebug = apitools.isFullDebugMode(req);

    if (!apitools.validateSchema(req, res, apidef))
        return;

    //------------------------
    logger.logger.info('IngenicoPrint - preparing');
    var printoutConverted = []; //zamieniam bool na int
    req.body.printout.forEach(pp => {
        printoutConverted.push({
            type: pp.type,
            align: pp.align,
            bold: (pp.bold ? 1 : 0),
            high: (pp.high ? 1 : 0),
            expand: (pp.expand ? 1 : 0),
            text: pp.text
        })
    });

    const isconnected = apitools.IsConnected();
    if (isconnected) {

        //specjalnie nie przekazuje req.body.transport, zeby metoda niepotrzebnie nie wysylala pos_hello
        scheduleCommand({
            type: 'PolcardIngenicoPrint',
            attributes: [fulldebug ? "1" : "0", {
                messageNumber: req.body.messageNumber,
                printout: printoutConverted,
                file: apitools.GetLicenseFile()
            }]
        });
        return apitools.sendResponse(req, res, { ok: true });

    } else {
        apitools.ConnectionBegin(req, (err) => {
            if (!err.ok) {
                apitools.sendErrorResponse(req, res, err);
                return;
            }

            //console.log('aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa');
            const msg0 = nativeTest.PolcardIngenicoPrint(fulldebug ? "1" : "0", {
                messageNumber: req.body.messageNumber,
                transport: {
                    encryptSession: apitools.isBooleanTrue(req.body.transport.encryptSession) ? 1 : 0,
                    encryptAlgorithm: req.body.transport.encryptAlgorithm ? req.body.transport.encryptAlgorithm : '',
                    sessionKeyLen: req.body.transport.sessionKeyLen ? req.body.transport.sessionKeyLen : 0,
                    publicKey: req.body.transport.publicKey ? req.body.transport.publicKey : ''
                },
                printout: printoutConverted,
                file: apitools.GetLicenseFile()
            });
            logger.logger.info('IngenicoPrint done: ' + JSON.stringify(msg0));
            if (!verifyTermResponse(req, res, msg0)) return;

            if (!msg0.ok)
                apitools.sendErrorResponse(req, res, msg0);
            else
                apitools.sendResponse(req, res, msg0);

            apitools.ConnectionEnd();
        });
    }
}

function processIngenicoPromptBarcode(req, res) {
    //req.body['type'] = 'barcode'; type na null to inaczej barcode
    return processIngenicoPrompt(req, res);
}

function processIngenicoPrompt(req, res) {

    var fulldebug = apitools.isFullDebugMode(req);

    if (!apitools.validateSchema(req, res, apidef))
        return;

    if (req.body.type == null && req.body.barcode != null )
        req.body['type'] = 'barcode'; //type na null to inaczej barcode

    //------------------------
    logger.logger.info('IngenicoPrompt - preparing');

    const isconnected = apitools.IsConnected();
    if (isconnected) {

        //specjalnie nie przekazuje req.body.transport, zeby metoda niepotrzebnie nie wysylala pos_hello
        scheduleCommand({
            type: 'PolcardIngenicoPrompt',
            attributes: [fulldebug ? "1" : "0", {
                type: req.body.type,
                reqestParams: req.body.reqestParams ? req.body.reqestParams : '',
                barcode: req.body.barcode ? req.body.barcode : '',
                headerText: req.body.headerText ? req.body.headerText : '',
                bodyText: req.body.bodyText ? req.body.bodyText : '',
                time: req.body.time ? req.body.time : 0,
                file: apitools.GetLicenseFile()
            }]
        });
        return apitools.sendResponse(req, res, { ok: true });

    } else {
        apitools.ConnectionBegin(req, async (err) => {
            if (!err.ok) {
                apitools.sendErrorResponse(req, res, err);
                return;
            }

            //console.log('aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa');
            const msg0 = nativeTest.PolcardIngenicoPrompt(fulldebug ? "1" : "0", {
                type: req.body.type,
                reqestParams: req.body.reqestParams ? req.body.reqestParams:'',
                barcode: req.body.barcode ? req.body.barcode: '',
                headerText: req.body.headerText ? req.body.headerText:'',
                bodyText: req.body.bodyText ? req.body.bodyText:'',
                time: req.body.time ? req.body.time:0,
                transport: {
                    encryptSession: apitools.isBooleanTrue(req.body.transport.encryptSession) ? 1 : 0,
                    encryptAlgorithm: req.body.transport.encryptAlgorithm ? req.body.transport.encryptAlgorithm : '',
                    sessionKeyLen: req.body.transport.sessionKeyLen ? req.body.transport.sessionKeyLen : 0,
                    publicKey: req.body.transport.publicKey ? req.body.transport.publicKey : ''
                },
                file: apitools.GetLicenseFile()
            });
            logger.logger.info('IngenicoPrompt done: ' + JSON.stringify(msg0));
            if (!verifyTermResponse(req, res, msg0)) return;

            if (!msg0.ok)
                apitools.sendErrorResponse(req, res, msg0);
            else
                apitools.sendResponse(req, res, msg0);

            logger.logger.info('IngenicoPrompt listening ....');
            var msg1 = await startListeningForPromptEnd(req, res, fulldebug);
            logger.logger.info('finished waiting; msg1 done: ' + JSON.stringify(msg1));

            apitools.ConnectionEnd();
        });
    }
}

function processIngenicoSetAction(req, res)
{
    var fulldebug = apitools.isFullDebugMode(req);

    if (!apitools.validateSchema(req, res, apidef))
        return;

    apitools.ConnectionBegin(req, async (err) => {
        if (!err.ok) {
            apitools.sendErrorResponse(req, res, err);
            return;
        }

        //console.log('aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa');
        const msg0 = nativeTest.PolcardIngenicoSetAction(fulldebug ? "1" : "0", {
            action: req.body.action,
            transport: {
                encryptSession: apitools.isBooleanTrue(req.body.transport.encryptSession) ? 1 : 0,
                encryptAlgorithm: req.body.transport.encryptAlgorithm ? req.body.transport.encryptAlgorithm : '',
                sessionKeyLen: req.body.transport.sessionKeyLen ? req.body.transport.sessionKeyLen : 0,
                publicKey: req.body.transport.publicKey ? req.body.transport.publicKey : ''
            },
            file: apitools.GetLicenseFile()
        });
        logger.logger.info('IngenicoSetAction done: ' + JSON.stringify(msg0));
        if (!verifyTermResponse(req, res, msg0)) return;

        if (!msg0.ok)
            apitools.sendErrorResponse(req, res, msg0);
        else
            apitools.sendResponse(req, res, msg0);

        //jelsi close_business_day to czekam na <pinpad_cutoff_finish > 2880300
        //if (req.body.action == 'close_business_day') {
        //    logger.logger.info('IngenicoSetAction listening ....');
        //    var msg1 = await startListeningForCutoffFinish(req, res, fulldebug);
        //    //var msg1 = await startListeningForever(req, res, fulldebug);
        //    logger.logger.info('finished waiting; msg1 done: ' + JSON.stringify(msg1));
        //}

        apitools.ConnectionEnd();
    });
}

function processIngenicoCutoff(req, res) {
    var fulldebug = apitools.isFullDebugMode(req);

    if (!apitools.validateSchema(req, res, apidef))
        return;

    apitools.ConnectionBegin(req, async (err) => {
        if (!err.ok) {
            apitools.sendErrorResponse(req, res, err);
            return;
        }

        //console.log('aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa');
        const msg0 = nativeTest.PolcardIngenicoCutoff(fulldebug ? "1" : "0", {
            transport: {
                encryptSession: apitools.isBooleanTrue(req.body.transport.encryptSession) ? 1 : 0,
                encryptAlgorithm: req.body.transport.encryptAlgorithm ? req.body.transport.encryptAlgorithm : '',
                sessionKeyLen: req.body.transport.sessionKeyLen ? req.body.transport.sessionKeyLen : 0,
                publicKey: req.body.transport.publicKey ? req.body.transport.publicKey : ''
            },
            file: apitools.GetLicenseFile()
        });
        logger.logger.info('IngenicoCutoff done: ' + JSON.stringify(msg0));
        if (!verifyTermResponse(req, res, msg0)) return;

        if (!msg0.ok)
            apitools.sendErrorResponse(req, res, msg0);
        else
            apitools.sendResponse(req, res, msg0);

        //jelsi close_business_day to czekam na <pinpad_cutoff_finish > 2880300
        logger.logger.info('IngenicoSetAction listening ....');
        var msg1 = await startListeningForCutoffFinish(req, res, fulldebug);
        //var msg1 = await startListeningForever(req, res, fulldebug);
        logger.logger.info('finished waiting; msg1 done: ' + JSON.stringify(msg1));

        apitools.ConnectionEnd();
    });
}

function processIngenicoCutoffReport(req, res) {
    var fulldebug = apitools.isFullDebugMode(req);

    if (!apitools.validateSchema(req, res, apidef))
        return;

    apitools.ConnectionBegin(req, async (err) => {
        if (!err.ok) {
            apitools.sendErrorResponse(req, res, err);
            return;
        }

        //console.log('aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa');
        const msg0 = nativeTest.PolcardIngenicoCutoffReport(fulldebug ? "1" : "0", {
            transport: {
                encryptSession: apitools.isBooleanTrue(req.body.transport.encryptSession) ? 1 : 0,
                encryptAlgorithm: req.body.transport.encryptAlgorithm ? req.body.transport.encryptAlgorithm : '',
                sessionKeyLen: req.body.transport.sessionKeyLen ? req.body.transport.sessionKeyLen : 0,
                publicKey: req.body.transport.publicKey ? req.body.transport.publicKey : ''
            },
            file: apitools.GetLicenseFile()
        });
        logger.logger.info('IngenicoCutoffReport done: ' + JSON.stringify(msg0));
        if (!verifyTermResponse(req, res, msg0)) return;

        if (!msg0.ok)
            apitools.sendErrorResponse(req, res, msg0);
        else
            apitools.sendResponse(req, res, msg0);

        //jelsi close_business_day to czekam na <pinpad_cutoff_finish > 2880300
        //logger.logger.info('IngenicoSetAction listening ....');
        //var msg1 = await startListeningForCutoffFinish(req, res, fulldebug);
        //var msg1 = await startListeningForever(req, res, fulldebug);
        //logger.logger.info('finished waiting; msg1 done: ' + JSON.stringify(msg1));

        apitools.ConnectionEnd();
    });
}



function processIngenicoReport(req, res) {
    var fulldebug = apitools.isFullDebugMode(req);

    if (!apitools.validateSchema(req, res, apidef))
        return;

    apitools.ConnectionBegin(req, async (err) => {
        if (!err.ok) {
            apitools.sendErrorResponse(req, res, err);
            return;
        }

        //console.log('aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa');
        const msg0 = nativeTest.PolcardIngenicoReport(fulldebug ? "1" : "0", {
            transport: {
                encryptSession: apitools.isBooleanTrue(req.body.transport.encryptSession) ? 1 : 0,
                encryptAlgorithm: req.body.transport.encryptAlgorithm ? req.body.transport.encryptAlgorithm : '',
                sessionKeyLen: req.body.transport.sessionKeyLen ? req.body.transport.sessionKeyLen : 0,
                publicKey: req.body.transport.publicKey ? req.body.transport.publicKey : ''
            },
            reportType: req.body.reportType,
            action: req.body.action,
            profileId: req.body.profileId,
            timeout: req.body.timeout,
            file: apitools.GetLicenseFile()
        });
        logger.logger.info('IngenicoReport done: ' + JSON.stringify(msg0));
        if (!verifyTermResponse(req, res, msg0)) return;

        if (!msg0.ok)
            apitools.sendErrorResponse(req, res, msg0);
        else
            apitools.sendResponse(req, res, msg0);


        //"reportType": "batch_report",
        //"action": "manual_batch_then_return",
        if (req.body.action == 'manual_batch_then_return') { // -- keep listening
            //jelsi close_business_day to czekam na <pinpad_cutoff_finish > 2880300
            //logger.logger.info('IngenicoSetAction listening ....');
            var msg1 = await startListeningForReportFinish(req, res, fulldebug);  //POLCARD_PINPAD_REPORT_FINISH
            //var msg1 = await startListeningForever(req, res, fulldebug);
            logger.logger.info('finished waiting; msg1 done: ' + JSON.stringify(msg1));
        }
        apitools.ConnectionEnd();
    });
}

function processIngenicoBarcode(req, res)
{
    var fulldebug = apitools.isFullDebugMode(req);

    if (!apitools.validateSchema(req, res, apidef))
        return;

    apitools.ConnectionBegin(req, async (err) => {
        if (!err.ok) {
            apitools.sendErrorResponse(req, res, err);
            return;
        }

        //console.log('aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa');
        const msg0 = nativeTest.PolcardIngenicoBarcode(fulldebug ? "1" : "0", {
            action: req.body.action,
            //transport: req.body.transport,
            file: apitools.GetLicenseFile()
        });
        logger.logger.info('IngenicoBarcode done: ' + JSON.stringify(msg0));
        if (!verifyTermResponse(req, res, msg0)) return;

        if (!msg0.ok)
            apitools.sendErrorResponse(req, res, msg0);
        else
            apitools.sendResponse(req, res, msg0);

        //logger.logger.info('IngenicoPrompt listening ....');
        //var msg1 = await startListeningForPromptEnd(req, res, fulldebug);
        //logger.logger.info('finished waiting; msg1 done: ' + JSON.stringify(msg1));

        apitools.ConnectionEnd();
    });
}

function processIngenicoPromptEnd(req, res)
{
    var fulldebug = apitools.isFullDebugMode(req);
    var resetstate = apitools.isResetStateSet(req);

    var trEnd = nativeTest.PolcardIngenicoPromptEnd(fulldebug ? "1" : "0", resetstate ? "1" : "0");
    //if (!verifyTermResponse(req, res, trEnd)) return;
    logger.logger.info('trEnd done: ' + JSON.stringify(trEnd));

    return apitools.sendResponse(req, res, trEnd);
}

function processIngenicoCutoffEnd(req, res) {
    var fulldebug = apitools.isFullDebugMode(req);
    var resetstate = apitools.isResetStateSet(req);

    var trEnd = nativeTest.PolcardIngenicoCutoffEnd(fulldebug ? "1" : "0", resetstate ? "1" : "0");
    //if (!verifyTermResponse(req, res, trEnd)) return;
    logger.logger.info('trEnd done: ' + JSON.stringify(trEnd));

    return apitools.sendResponse(req, res, trEnd);
}
function processIngenicoReportEnd(req, res) {
    var fulldebug = apitools.isFullDebugMode(req);
    var resetstate = apitools.isResetStateSet(req);

    var trEnd = nativeTest.PolcardIngenicoReportEnd(fulldebug ? "1" : "0", resetstate ? "1" : "0");
    //if (!verifyTermResponse(req, res, trEnd)) return;
    logger.logger.info('trEnd done: ' + JSON.stringify(trEnd));

    return apitools.sendResponse(req, res, trEnd);
}

async function goAndListen(req, res) {
    var fulldebug = apitools.isFullDebugMode(req);

    apitools.ConnectionBegin(req, async (err) => {
        if (!err.ok) {
            apitools.sendErrorResponse(req, res, err);
            return;
        }


        apitools.sendResponse(req, res, {});
        //if (!msg0.ok)
        //    apitools.sendErrorResponse(req, res, msg0);
        //else
        //    apitools.sendResponse(req, res, msg0);

        //jelsi close_business_day to czekam na <pinpad_cutoff_finish > 2880300
        logger.logger.info('listening ....');
        var msg1 = await startListeningForever(req, res, fulldebug);
        logger.logger.info('finished waiting; msg1 done: ' + JSON.stringify(msg1));

        apitools.ConnectionEnd();
    });

    var msg1 = await startListeningForever(req, res, fulldebug);
    logger.logger.info('finished waiting; msg1 done: ' + JSON.stringify(msg1));
}
function processIngenicoListen(req, res) {
    //var fulldebug = apitools.isFullDebugMode(req);
    goAndListen(req, res);
}


function sleep(ms) {
    return new Promise((resolve) => {
        setTimeout(resolve, ms);
    });
}

async function startListeningForever(req, res, fulldebug) {
    var trStatus = {};
    do {
        logger.logger.info('KeepListeningForever ..........');
        trStatus = nativeTest.PolcardTransactionKeepListeningForever(fulldebug ? "1" : "0");
        logger.logger.info('KeepListeningForever done (continue if ok=true): ' + JSON.stringify(trStatus));

        var shouldContinue = trStatus.ok;
        if (!shouldContinue) {
            break;
        }
        await sleep(500);
    } while (1);

    return trStatus;
}

async function startListeningForPromptEnd(req, res, fulldebug) {
    readingFromTransactionInProgress = true;
    var trStatus = {};
    do {
        logger.logger.info('KeepListeningForPromptEnd ..........');
        trStatus = nativeTest.PolcardTransactionKeepListeningForPromptEnd(fulldebug ? "1" : "0");
        logger.logger.info('KeepListeningForPromptEnd done (continue if ok=true): ' + JSON.stringify(trStatus));

        processScheduledCommands();

        var shouldContinue = trStatus.ok;
        if (!shouldContinue) {
            readingFromTransactionInProgress = false;
            break;
        }
        await sleep(500);
    } while (1);

    return trStatus;
}

async function startListeningForCutoffFinish(req, res, fulldebug) {
    readingFromTransactionInProgress = true;
    var trStatus = {};
    do {
        logger.logger.info('KeepListeningForCutoffFinish ..........');

        //ta metoda wola WaitForExtraBytesFromTerminal jesli ostania pozycja z currentTransactionStatuses_ to POLCARD_PINPAD_WAIT, POLCARD_PINPAD_DISPLAY lub POLCARD_PINPAD_STOP_DISPLAY
        //Funkcja PolcardTransactionKeepListeningForCutoffFinish zwroci ok = false kiedy ostatnim message w currentTransactionStatuses_ jest POLCARD_PINPAD_CUTOFF_FINISH /*2880300*/
        trStatus = nativeTest.PolcardTransactionKeepListeningForCutoffFinish(fulldebug ? "1" : "0");
        logger.logger.info('KeepListeningForCutoffFinish done (continue if ok=true): ' + JSON.stringify(trStatus));

        processScheduledCommands();

        var shouldContinue = trStatus.ok;
        if (!shouldContinue) {
            readingFromTransactionInProgress = false;
            break;
        }
        await sleep(100);
    } while (1);

    return trStatus;
}

async function startListeningForReportFinish(req, res, fulldebug) {
    readingFromTransactionInProgress = true;
    var trStatus = {};
    do {
        logger.logger.info('KeepListeningForReportFinish ..........');
        trStatus = nativeTest.PolcardTransactionKeepListeningForReportFinish(fulldebug ? "1" : "0");
        logger.logger.info('KeepListeningForReportFinish done (continue if ok=true): ' + JSON.stringify(trStatus));
        //console.log('KeepListeningForReportFinish done');
        processScheduledCommands();
        //console.log('processScheduledCommands done');

        var shouldContinue = trStatus.ok;
        if (!shouldContinue) {
            readingFromTransactionInProgress = false;
            break;
        }
        await sleep(100);
    } while (1);

    return trStatus;
}

function processScheduledCommands() {
    if (scheduledCommands_.length > 0) {
        var cmds = scheduledCommands_;
        scheduledCommands_ = [];
        cmds.forEach(obj => {
            if (obj.type == 'PolcardTransactionDataInput') {
                const trReconInfo = nativeTest.PolcardTransactionDataInput(obj.attributes[0], obj.attributes[1]);  //jesli jestem wtrakcie transackji to terminal akceptuje bufor ale i tak nie odpowaida zadnym innym message
                logger.logger.info('scheduled message done: ' + JSON.stringify(trReconInfo));
            }
            if (obj.type == 'PolcardIngenicoPrint') {
                const trReconInfo = nativeTest.PolcardIngenicoPrint(obj.attributes[0], obj.attributes[1]);
                logger.logger.info('scheduled message done: ' + JSON.stringify(trReconInfo));
            }
            if (obj.type == 'PolcardIngenicoPrompt') {
                const trReconInfo = nativeTest.PolcardIngenicoPrompt(obj.attributes[0], obj.attributes[1]);
                logger.logger.info('scheduled message done: ' + JSON.stringify(trReconInfo));
            }


        });
    }
}

async function startListeningForWaitTrEnd(req, res, fulldebug) {

    readingFromTransactionInProgress = true;
    var trStatus = {};
    //var sentOnce = false;
    var timeoutCounter = 0;
    do {
        trStatus = nativeTest.PolcardTransactionKeepListeningForPosFinish(fulldebug ? "1" : "0");
        logger.logger.info('KeepListening WaitTrEnd (continue if ok=true): ' + JSON.stringify(trStatus));
        //{"message":"msg2 done: {\"ok\":true,\"error\":\"\",\"terminal\":{\"returnCode\":0,\"error\":\"Czy uzyc platnosci mobilnej?\",\"display\":\"cardholder_and_clerk\",\"keyboardOptions\":\"yes_no\",\"timeout\":120}}

        processScheduledCommands();

        //if (!sentOnce) {
        //if (trStatus.ok && trStatus.terminal && trStatus.terminal.display == 'cardholder_and_clerk') {
        //    const replyType = 1;
        //    const keycode = "0"
        //    const trReconInfo = nativeTest.PolcardTransactionDataInput(fulldebug ? "1" : "0", { text: keycode, type: replyType, waitForResponse: 0 });  //jesli jestem wtrakcie transackji to terminal akceptuje bufor ale i tak nie odpowaida zadnym innym message
        //    console.log(JSON.stringify(trReconInfo));
        //
        //    sentOnce = true;
        //}
        var shouldContinue = trStatus.ok;
        if (!shouldContinue) {
            readingFromTransactionInProgress = false;
            break;
        }
        await sleep(100);
    } while(1);

    return trStatus;
}

function initModule(anativeTest, aapidef) {
    nativeTest = anativeTest;
    apidef = aapidef;
}


module.exports.initModule = initModule;
module.exports.ingenicoTransactionSchema = ingenicoTransactionSchema;
module.exports.ingenicoTransportSchema = ingenicoTransportSchema;



module.exports.processIngenicoTransactionStart = processIngenicoTransactionStart;
module.exports.processPolcardTransactionStatus = processPolcardTransactionStatus;
module.exports.processPolcardTransactionStatuses = processPolcardTransactionStatuses;

module.exports.processPolcardDisplayInfo = processPolcardDisplayInfo;
module.exports.processIngenicoTransactionDataInput = processIngenicoTransactionDataInput;
module.exports.processIngenicoTransactionSendKey = processIngenicoTransactionSendKey;
module.exports.processIngenicoTransactionSendCode = processIngenicoTransactionSendCode;
module.exports.processIngenicoTransactionSendAmount = processIngenicoTransactionSendAmount;
module.exports.processIngenicoTransactionSendOption = processIngenicoTransactionSendOption;
module.exports.processIngenicoTransactionSendProfile = processIngenicoTransactionSendProfile;


module.exports.processIngenicoPrint = processIngenicoPrint;
module.exports.processIngenicoPrompt = processIngenicoPrompt;
module.exports.processIngenicoPromptBarcode = processIngenicoPromptBarcode;
module.exports.processIngenicoBarcode = processIngenicoBarcode;

module.exports.processIngenicoSetAction = processIngenicoSetAction;

module.exports.processIngenicoCutoff = processIngenicoCutoff;
module.exports.processIngenicoCutoffEnd = processIngenicoCutoffEnd;
module.exports.processIngenicoReportEnd = processIngenicoReportEnd;
module.exports.processIngenicoCutoffReport = processIngenicoCutoffReport;

module.exports.processIngenicoPromptEnd = processIngenicoPromptEnd;

module.exports.processIngenicoReport = processIngenicoReport;

module.exports.processIngenicoListen = processIngenicoListen;



module.exports.processPolcardIngenicoTransactionEnd = processPolcardIngenicoTransactionEnd;
module.exports.processPolcardIngenicoTransactionAbort = processPolcardIngenicoTransactionAbort;

module.exports.processPolcardIngenicoTransactionLast = processPolcardIngenicoTransactionLast;
