'use strict';

const os = require('os');
const url = require('url');
const request = require('request');
const fs = require("fs");
const async = require('async');
//const request = require('request');

var nativeTest = null;  //inicjalizowane  wetodzie initModule()

const yaml_config = require('node-yaml-config');
const logger = require('./logger');

//###################################################
//                  Config
//###################################################
logger.logger.info('Loading config, make sure config.yml has valid structure');
//var buildVersion = null;
var config = yaml_config.load(__dirname + '/config.yml');
logger.logger.info('config: DONE');

function isBooleanTrue(value) {
    if (value == null) return false;
    if (value == "false") return false;
    if (value == false) return false;
    if (value == "true") return true;
    if (value == true) return true;
    return false;
}

function isFullDebugMode(req) {
    if (req == null) return false;
    if (req.query == null) return false;
    if (req.query.fulldebug == null) return false;
    if (req.query.fulldebug == "false") return false;

    if (req.query.fulldebug == "true") return true;

    return false;

    //return req.query.fulldebug != null ? (req.query.fulldebug ? true : false) : false;
}
function isResetStateSet(req) {
    if (req == null) return false;
    if (req.query == null) return false;
    if (req.query.resetstate == null) return false;
    if (req.query.resetstate == "false") return false;
    if (req.query.resetstate == "true") return true;
    return false;
}

function getSchemaForRequest(req, api_def) {
    //console.log(req);
    let parsedUrl = url.parse(req.url);
    return api_def[parsedUrl.pathname] != null ? api_def[parsedUrl.pathname].schema : null;
}

function validateSchema(req, res, api_def) {
    var schema = getSchemaForRequest(req, api_def);
    if (schema != null) {
        var test = schema.validate(req.body);
        if (test.error != null) {
            logger.logger.warn('Payload validation error:' + JSON.stringify(test));
            sendErrorResponse(req, res, test.error);
            return false;
        } else {
            return true;
        }
    } else {
        return true;    //no validation - everythign is fine :-)
    }
}

function isPrettyOutput(req) {
    return req.query != null && req.query.pretty;
}
function prettyJson(obj) {
    return '<tt>' + JSON.stringify(obj, null, 2).replace(/\n/g, "<br/>").replace(/ /g, "&nbsp;") + '</tt>';
}

function sendErrorResponse(req, res, resultobj) {
    var watermark = GetWatermarkString();
    if (resultobj === Object(resultobj)) {
        resultobj['watermark'] = watermark;
    }

    if (isPrettyOutput(req))
        return res.send(prettyJson(resultobj /*{
            ok: false,
            terminal: resultobj
        }*/));
    else
        return res.json(resultobj /*{
            ok: false,
            terminal: resultobj
        }*/);
}

function sendResponse(req, res, resultobj) {
    var watermark = GetWatermarkString();
    if (resultobj === Object(resultobj)) {
        resultobj['watermark'] = watermark;
    }

    if (isPrettyOutput(req))
        return res.send(prettyJson(resultobj/*{
            ok: true,
            terminal: resultobj
        }*/));
    else
        return res.json(resultobj /*{
            ok: true,
            terminal: resultobj
        }*/);
}

function IsConnected() {
    return nativeTest.PosnetIsConnected().ok;
}

function ConnectionBegin(req, finalcb) {

    var fulldebug = isFullDebugMode(req);
    //FetchRemoteConfig(req, res, (err) => {
    //    if (err) {
    //        logger.logger.warn('FetchRemoteConfig error: ' + err);
    //        finalcb(true);  //return error
    //        return;
    //    }

    //    ValidateScurityToken(req, res, (result, err) => {
    //        if (err) {          //communication error
    //            finalcb(true);  //return error
    //        }
    //        else if (result == false) {     //validation didnt pass
    //            logger.logger.warn('Secutity token doesn\'t match');
    //            tools.resp_error(res, 'Secutity token doesn\'t match');
    //            finalcb(true);  //return error
    //        }
    //        else {
    if (nativeTest.PosnetIsConnected().ok) {
        finalcb({ ok: false, error: 'Already connected - check connect sequence or another transaction is in progress' });
        //already connected
        //finalcb({ ok: true, error: 'Already connected' });
    } else {
        if (fulldebug) logger.logger.info('FULLDEBUG: Preparing connection to the printer');
        var rc = ConnectionBeginProcess(req);
        finalcb(rc);
    }
    //        }
    //   });
    //});

}


function ConnectionBeginProcess(req) {
    var fulldebug = isFullDebugMode(req);
    var stat = null;
    if (config.ingenico.connection.type == 'rs232') {
        if (fulldebug) logger.logger.info('FULLDEBUG: Opening COM port ' + config.ingenico.connection.rs232.params);
        const disable_ioctl = config.ingenico.connection.rs232.disable_ioctl != null ? config.ingenico.connection.rs232.disable_ioctl : false;
        stat = nativeTest.PosnetConnectRS232(config.ingenico.connection.rs232.params, fulldebug ? "1" : "0", disable_ioctl ? "1" : "0");
        if (fulldebug) logger.logger.info('FULLDEBUG: Opening COM port result' + JSON.stringify(stat));
    } else if (config.ingenico.connection.type == 'eth') {
        if (fulldebug) logger.logger.info('FULLDEBUG: Opening Socket ' + config.ingenico.connection.eth.ip + ':' + config.ingenico.connection.eth.port);
        stat = nativeTest.PosnetConnectEth(config.ingenico.connection.eth.ip, config.ingenico.connection.eth.port, fulldebug ? "1" : "0");
    }

    if (stat == null || stat.ok == false) {
        logger.logger.error(JSON.stringify(stat));
        //tools.resp_error(res, stat.error);
        return (stat == null ? { ok: false, error: 'Unknown error' } : stat);
    } else {
        return stat;
    }
}

function ConnectionEnd() {
    var stat = nativeTest.PosnetDisconnect();
    if (!stat.ok) {
        logger.logger.error(JSON.stringify(stat));
        //tools.resp_error(res, stat.error);
    }
    return stat;
}

function readStatInfo(stat) {
    var licenseTo = '';
    var validTo = 0;
    var expired = true;
    stat.results.forEach(result => {
        if (result.licenseTo != null && result.licenseTo != '')
            licenseTo = result.licenseTo;
        if (result.validTo != null && result.validTo != '')
            validTo = result.validTo;
        if (result.expired != null && result.expired != '')
            expired = result.expired == 'false' ? false : true;
    });
    return {
        licenseTo: licenseTo,
        validTo: validTo,
        expired: expired
    };
}

function obtainNewLicenseFile(uniqueLicenseId) {
    var fullurl = 'https://license.bigdotsoftware.pl/obtain/' + uniqueLicenseId;
    logger.logger.debug(fullurl);
    request({
        url: fullurl,
        method: 'POST',
        rejectUnauthorized: false,
        strictSSL: false,
        encoding: null /* body as buffer*/,
        json: {
            'action': 'renew'
        }
    }, (err, internalres, body) => {
        logger.logger.info('statusCode:', internalres && internalres.statusCode);
        if (err)
            logger.logger.info('cannot execute license endpoint: ' + err);
        else if (internalres && internalres.statusCode != 200)
            logger.logger.error('cannot execute license endpoint, statusCode: ' + internalres.statusCode);
        else {
            logger.logger.info('license endpoint executed ok');
            var fileDestination = config.license.file + '.' + new Date().getTime() + '.backup';
            logger.logger.info('license endpoint copy as ' + fileDestination);

            fs.copyFile(config.license.file, fileDestination, (err) => {
                if (err) {
                    logger.logger.error('error ' + err);
                } else {
                    logger.logger.info('copied as ' + fileDestination);
                    fs.writeFileSync(config.license.file + '.new', body, 'binary');

                    logger.logger.info('validating ' + config.license.file + '.new');
                    var stat = nativeTest.ValidateLicenseFile({ file: config.license.file + '.new', skipcache: 1 });
                    logger.logger.info('validated' + JSON.stringify(stat));
                    if (stat.ok) {
                        logger.logger.info('downloaded file is valid license file');
                        var statinfo = readStatInfo(stat);
                        logger.logger.debug(statinfo);
                        if (!statinfo.expired) {
                            logger.logger.info('replacing license file....');
                            fs.copyFile(config.license.file + '.new', config.license.file, (err) => {
                                if (err)
                                    logger.logger.error('error replacing license file: ' + err);
                                else
                                    logger.logger.info('license file replaced');
                            });
                        } else {
                            logger.logger.warn('... but expired');
                        }
                    } else {
                        logger.logger.error('downloaded file is not valid license file');
                    }
                }
            });
        }
    });
}

function CheckLicense() {
    var stat = { ok: false };
    if (config.license.file == null || config.license.file == '') {
        logger.logger.warn('No license file provided. Amount on terminal in unregistered version is randomly increased by PLN grosze.');
        logger.logger.warn('Brak pliku licencji. W wersji nielicencjonowanej do kwoty na terminalu dodawane sa losowe grosze.');
    } else {
        //if (config.license.url != null && config.license.url != '') {
        // TODO
        //}
        if (config.license.file != null && config.license.file != '') {
            logger.logger.info('license file: ' + config.license.file);
            stat = nativeTest.ValidateLicenseFile({ file: config.license.file });
            logger.logger.info(JSON.stringify(stat));
            if (!stat.ok) {
                logger.logger.warn('Wrong license file provided. Amount on terminal in unregistered version is randomly increased by PLN grosze.');
                logger.logger.warn('Bdny plik licencji. W wersji nielicencjonowanej do kwoty na terminalu dodawane sa losowe grosze.');
            }

            if (stat.results != null) {
                var uniqueLicenseId = '';
                var days = 999;
                stat.results.forEach(result => {
                    if (result.validTo != null && result.validTo != '') {
                        //logger.logger.debug(parseInt(result.validTo));
                        var seconds = parseInt(result.validTo) - new Date().getTime() / 1000;
                        days = seconds / 60 / 60 / 24;
                        if (days < 5)
                            logger.logger.error(days + ' days left');
                        else if (days < 10)
                            logger.logger.warn(days + ' days left');
                        else
                            logger.logger.info(days + ' days left');
                    }

                    if (result.uniqueLicenseId != null && result.uniqueLicenseId != '') {
                        uniqueLicenseId = result.uniqueLicenseId;
                    }
                });

                //przedstaw sie jakiej licencji uzywasz i gdzie

                var ifaces = os.networkInterfaces();
                var ifacesarray = [];
                Object.keys(ifaces).forEach(function (ifname) {
                    var alias = 0;

                    ifaces[ifname].forEach(function (iface) {
                        if ('IPv4' !== iface.family || iface.internal !== false) {
                            // skip over internal (i.e. 127.0.0.1) and non-ipv4 addresses
                            return;
                        }

                        if (alias >= 1) {
                            // this single interface has multiple ipv4 addresses
                            //console.log(ifname + ':' + alias, iface.address);
                            ifacesarray.push({
                                name: ifname + ':' + alias,
                                address: iface.address
                            });
                        } else {
                            // this interface has only one ipv4 adress
                            //console.log(ifname, iface.address);
                            ifacesarray.push({
                                name: ifname,
                                address: iface.address
                            });
                        }
                        ++alias;
                    });
                });

                request({
                    url: 'http://bigdotsoftware.pl/inviteposnetserver.php',
                    method: 'POST',
                    rejectUnauthorized: false,
                    strictSSL: false,
                    json: {
                        'action': 'license',
                        'days': days,
                        'uniqueLicenseId': uniqueLicenseId,
                        'hostname': os.hostname(),
                        'platform': os.platform(),
                        'arch': os.arch(),
                        'uptime': os.uptime(),
                        'type': os.type(),
                        'ifacesarray': ifacesarray
                    }
                }, (err, internalres, body) => {
                    if (err)
                        logger.logger.info('cannot execute invitation: ' + err);
                    else
                        logger.logger.info('executed ok');
                });

                if (uniqueLicenseId != '' && days < 10) {
                    logger.logger.info('Obtaining new license file');
                    obtainNewLicenseFile(uniqueLicenseId);
                }

            }
        }
    }
    return stat;
}

function GetLicenseFile() {
    return config.license.file;
}

function GetWatermarkString() {

    var stat = CheckLicense();
    if (stat.ok) {
        var statinfo = readStatInfo(stat);
        return 'Wersja licencjonowana dla ' + statinfo.licenseTo + ' wazna do ' + new Date(statinfo.validTo * 1000).toISOString();
    } else {
        return 'Wersja nielicencjonowana' + (config.license.file != '' ? ' lub przeznaczona dla innego produktu' : '') + '. W wersji nielicencjonowanej do kwoty na terminalu dodawane sa losowe grosze';
    }
}

function sendToPosnetServer(posnetserverObj) {
    logger.logger.warn(`Integracja z PosnetServer : ` + JSON.stringify(posnetserverObj));
    ExecuteUrl(config.posnetserver.url + '/paragon', 'POST', posnetserverObj);
}

function ExecuteUrl(fullurl, resultsurlmethod, body) {

    const tt_start = new Date().getTime();

    var apiMethodWrapper = function (callback) {
        request({
            url: fullurl,
            method: resultsurlmethod,
            rejectUnauthorized: false,
            strictSSL: false,
            json: body
        }, (err, internalres, body) => {
            if (err) {
                logger.logger.error('Cannot execute url: ' + fullurl + ' : ' + err);
                callback(true);//retry
            } else {
                logger.logger.debug(`URL ${fullurl} executed OK`);
                callback(false, {});//ok, no need to retry
            }
        });
    };

    async.retry({ times: config.ingenico.retry.times, interval: config.ingenico.retry.interval }, apiMethodWrapper, function (err, responseObj) {

        logger.logger.debug(`ExecuteUrl is Done`);
        const took = new Date().getTime() - tt_start;
        if (err) {
            logger.logger.error(`ExecuteUrl failed in ${took} ms : ${err}`);
        } else {
            logger.logger.info(`ExecuteUrl succeeded ${took} ms`);
        }

    });

}

function notifyBackendAboutTransactionResult(trEnd, callbackObj) {
    ExecuteUrl(callbackObj.uri, callbackObj.method, JSON.stringify(trEnd));
}

function initModule(anativeTest) {
    nativeTest = anativeTest;
}

module.exports.initModule = initModule;

module.exports.readStatInfo = readStatInfo;
module.exports.CheckLicense = CheckLicense;
module.exports.GetLicenseFile = GetLicenseFile;

module.exports.obtainNewLicenseFile = obtainNewLicenseFile;



module.exports.isBooleanTrue = isBooleanTrue;
module.exports.isFullDebugMode = isFullDebugMode;
module.exports.isResetStateSet = isResetStateSet;

module.exports.validateSchema = validateSchema;

module.exports.sendResponse = sendResponse;
module.exports.sendErrorResponse = sendErrorResponse;

module.exports.ConnectionEnd = ConnectionEnd;
module.exports.ConnectionBegin = ConnectionBegin;
module.exports.IsConnected = IsConnected;

module.exports.notifyBackendAboutTransactionResult = notifyBackendAboutTransactionResult;
module.exports.ExecuteUrl = ExecuteUrl;
module.exports.sendToPosnetServer = sendToPosnetServer;